from datetime import datetime
from gmo_fx.errors import ApiError


class Response:
    response_status: int
    response_time: datetime

    def __init__(self, response: dict):
        self.response_status = response["status"]
        self.response_time = datetime.strptime(
            response["responsetime"], "%Y-%m-%dT%H:%M:%S.%fZ"
        )
        if self.response_status != 0:
            err_code = response["messages"][0]["message_code"]
            message = response["messages"][0]["message_string"]
            raise ApiError(err_code, message=message)



import pytest
import re

from datetime import datetime
from gmo_fx.api.klines import KlinesApi, KlinesResponse
from tests.api_test_base import ApiTestBase
from typing import Callable, Literal, Optional
from unittest.mock import MagicMock, patch


class TestKlinesApi(ApiTestBase):

    def create_kline(
        self,
        open_time: str = "0000000000000",
        open: float = 0.0,
        high: float = 0.0,
        low: float = 0.0,
        close: float = 0.0,
    ) -> dict:
        return {
            "openTime": open_time,
            "open": str(open),
            "high": str(high),
            "low": str(low),
            "close": str(close),
        }

    def call_api(
        self,
        symbol: KlinesApi.Symbol,
        price_type: Literal["BID", "ASK"],
        interval: KlinesApi.KlineInterval,
        date: datetime,
    ) -> KlinesResponse:
        return KlinesApi()(
            symbol,
            price_type,
            interval,
            date,
        )

    def create_klines(
        self, size: int, kline_builder: Optional[Callable[[int], dict]] = None
    ) -> list[dict]:
        kline_builder = kline_builder or (lambda i: self.create_kline())
        return [kline_builder(i) for i in range(size)]

    @patch("gmo_fx.api.api_base.get")
    def test_klines_error(self, get_mock: MagicMock):
        self.check_404_error(
            get_mock,
            lambda: self.call_api(
                KlinesApi.Symbol.USD_JPY,
                "BID",
                KlinesApi.KlineInterval.Min1,
                datetime(2024, 1, 1),
            ),
        )

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_klines_from_response_klines_accesser(self, get_mock: MagicMock):
        expect_klines = [
            {
                "openTime": datetime(i + 1980, 1, 1),
                "open": float(i),
                "high": float(i),
                "low": float(i),
                "close": float(i),
            }
            for i in range(100)
        ]
        get_mock.return_value = self.create_response(
            data=self.create_klines(
                100,
                lambda i: self.create_kline(
                    open_time=str(int(expect_klines[i]["openTime"].timestamp()) * 1000),
                    open=expect_klines[i]["open"],
                    high=expect_klines[i]["high"],
                    low=expect_klines[i]["low"],
                    close=expect_klines[i]["close"],
                ),
            )
        )
        klines_response = self.call_api(
            KlinesApi.Symbol.USD_JPY,
            "BID",
            KlinesApi.KlineInterval.Min1,
            datetime(2024, 1, 1),
        )
        for i, expect in enumerate(expect_klines):
            assert klines_response.klines[i].open_time == expect["openTime"]
            assert klines_response.klines[i].open == expect["open"]
            assert klines_response.klines[i].high == expect["high"]
            assert klines_response.klines[i].low == expect["low"]
            assert klines_response.klines[i].close == expect["close"]

    def check_url_parameter(
        self,
        get_mock: MagicMock,
        symbol: KlinesApi.Symbol = KlinesApi.Symbol.USD_JPY,
        symbol_str: str = "USD_JPY",
        price_type: str = "ASK",
        price_type_str: str = "ASK",
        interval=KlinesApi.KlineInterval.Min1,
        interval_str="1min",
        date: datetime = datetime(2024, 1, 1),
        date_str: str = "20240101",
    ) -> None:

        get_mock.return_value = self.create_response(data=self.create_klines(1))
        self.call_api(symbol, price_type, interval, date)
        param_match = re.search("\?(.*)", get_mock.mock_calls[0].args[0])
        param = param_match.group(1)
        assert f"symbol={symbol_str}" in param
        assert f"priceType={price_type_str}" in param
        assert f"interval={interval_str}" in param
        assert f"date={date_str}" in param

    symbol_strs = [
        (KlinesApi.Symbol.USD_JPY, "USD_JPY"),
        (KlinesApi.Symbol.EUR_JPY, "EUR_JPY"),
        (KlinesApi.Symbol.GBP_JPY, "GBP_JPY"),
        (KlinesApi.Symbol.AUD_JPY, "AUD_JPY"),
        (KlinesApi.Symbol.NZD_JPY, "NZD_JPY"),
        (KlinesApi.Symbol.CAD_JPY, "CAD_JPY"),
        (KlinesApi.Symbol.CHF_JPY, "CHF_JPY"),
        (KlinesApi.Symbol.TRY_JPY, "TRY_JPY"),
        (KlinesApi.Symbol.ZAR_JPY, "ZAR_JPY"),
        (KlinesApi.Symbol.MXN_JPY, "MXN_JPY"),
        (KlinesApi.Symbol.EUR_USD, "EUR_USD"),
        (KlinesApi.Symbol.GBP_USD, "GBP_USD"),
        (KlinesApi.Symbol.AUD_USD, "AUD_USD"),
        (KlinesApi.Symbol.NZD_USD, "NZD_USD"),
    ]

    @pytest.mark.parametrize("symbol, symbol_str", symbol_strs)
    @patch("gmo_fx.api.api_base.get")
    def test_should_call_get_with_symbol(
        self, get_mock: MagicMock, symbol: KlinesApi.Symbol, symbol_str: str
    ):
        self.check_url_parameter(
            get_mock,
            symbol=symbol,
            symbol_str=symbol_str,
        )

    price_type_strs = [
        ("BID", "BID"),
        ("ASK", "ASK"),
    ]

    @pytest.mark.parametrize("price_type, price_type_str", price_type_strs)
    @patch("gmo_fx.api.api_base.get")
    def test_should_call_get_with_price_type(
        self, get_mock: MagicMock, price_type: str, price_type_str: str
    ):
        self.check_url_parameter(
            get_mock,
            price_type=price_type,
            price_type_str=price_type_str,
        )

    interval_strs = [
        (KlinesApi.KlineInterval.Min1, "1min"),
        (KlinesApi.KlineInterval.Min5, "5min"),
        (KlinesApi.KlineInterval.Min10, "10min"),
        (KlinesApi.KlineInterval.Min15, "15min"),
        (KlinesApi.KlineInterval.Min30, "30min"),
        (KlinesApi.KlineInterval.H1, "1hour"),
        (KlinesApi.KlineInterval.H4, "4hour"),
        (KlinesApi.KlineInterval.H8, "8hour"),
        (KlinesApi.KlineInterval.H12, "12hour"),
        (KlinesApi.KlineInterval.D1, "1day"),
        (KlinesApi.KlineInterval.W1, "1week"),
        (KlinesApi.KlineInterval.M1, "1month"),
    ]

    date_strs = [
        # datetime, interval, string
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.Min1, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.Min5, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.Min10, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.Min15, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.Min30, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.H1, "20240101"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.H4, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.H8, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.H12, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.D1, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.W1, "2024"),
        (datetime(2024, 1, 1, 10, 23, 12), KlinesApi.KlineInterval.M1, "2024"),
    ]

    @pytest.mark.parametrize("date, interval, string", date_strs)
    @patch("gmo_fx.api.api_base.get")
    def test_should_call_get_with_date(
        self,
        get_mock: MagicMock,
        date: datetime,
        interval: KlinesApi.KlineInterval,
        string: str,
    ):
        interval_map = dict(self.interval_strs)
        self.check_url_parameter(
            get_mock,
            interval=interval,
            interval_str=interval_map[interval],
            date=date,
            date_str=string,
        )

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=self.create_klines(1))
        self.call_api(
            KlinesApi.Symbol.USD_JPY, "BID", KlinesApi.KlineInterval.D1, datetime.now()
        )
        url_match = re.search("(.*)\?.*", get_mock.mock_calls[0].args[0])
        url = url_match.group(1)
        assert url == "https://forex-api.coin.z.com/public/v1/klines"



from dataclasses import dataclass
from datetime import datetime, timezone
from enum import Enum
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import SettleType, Side, Symbol


@dataclass
class Execution:
    Symbol = Symbol
    Side = Side
    SettleType = SettleType

    amount: float
    execution_id: int
    client_order_id: str
    order_id: int
    position_id: int
    symbol: Symbol
    side: Side
    settle_type: SettleType
    size: int
    price: float
    loss_gain: int
    fee: int
    settled_swap: float
    timestamp: datetime


class LatestExecutionsResponse(ResponseBase):
    executions: list[Execution]

    def __init__(self, response: dict):
        super().__init__(response)
        self.assets = []

        data = response["data"]["list"]
        self.executions = [
            Execution(
                amount=float(d["amount"]),
                execution_id=d["executionId"],
                client_order_id=d["clientOrderId"],
                order_id=d["orderId"],
                position_id=d["positionId"],
                symbol=Symbol(d["symbol"]),
                side=Side(d["side"]),
                settle_type=SettleType(d["settleType"]),
                size=int(d["size"]),
                price=float(d["price"]),
                loss_gain=int(d["lossGain"]),
                fee=int(d["fee"]),
                settled_swap=float(d["settledSwap"]),
                timestamp=datetime.strptime(
                    d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"
                ).replace(tzinfo=timezone.utc),
            )
            for d in data
        ]


class LatestExecutionsApi(PrivateApiBase):
    Symbol = Symbol

    @property
    def _path(self) -> str:
        return "latestExecutions"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return LatestExecutionsResponse

    def _api_error_message(self, response: Response):
        return (
            "最新約定一覧が取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def __call__(
        self,
        symbol: Symbol,
        count: int = 100,
    ) -> LatestExecutionsResponse:
        return super().__call__(path_query=f"symbol={symbol.value}" f"&count={count}")



from enum import auto, Enum
from typing import Type
from requests import get, Response
from gmo_fx.api.api_base import PublicApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.urls import BASE_URL_PUBLIC


class Status(Enum):
    """
    外国為替FXの稼動状態
    """

    OPEN = auto()  # オープン
    CLOSE = auto()  # クローズ
    MAINTENANCE = auto()  # メンテナンス

    @classmethod
    def from_str(cls, text: str) -> Type["Status"]:
        match text:
            case "OPEN":
                return cls.OPEN
            case "CLOSE":
                return cls.CLOSE
            case "MAINTENANCE":
                return cls.MAINTENANCE
        raise ValueError(f"不明なステータスです。: {text}")


class StatusResponse(ResponseBase):
    status: Status

    def __init__(self, response: dict):
        super().__init__(response)
        self.status = Status.from_str(response["data"]["status"])


class StatusApi(PublicApiBase):

    @property
    def _path(self) -> str:
        return f"status"

    @property
    def _method(self) -> PublicApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return StatusResponse

    def _api_error_message(self, response: Response):
        return (
            "ステータスが取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )



import json
import re
from datetime import datetime, timezone
from tests.api_test_base import ApiTestBase
from typing import Callable, Optional, Union
from unittest.mock import MagicMock, patch
from gmo_fx.api.change_oco_order import (
    Order,
    ChangeOcoOrderApi,
    ChangeOcoOrderResponse,
)


class TestChangeOcoOrderApi(ApiTestBase):

    def call_api(
        self,
        root_order_id: Optional[int] = None,
        client_order_id: Optional[str] = None,
        limit_price: Optional[float] = None,
        stop_price: Optional[float] = None,
    ) -> ChangeOcoOrderResponse:
        return ChangeOcoOrderApi(
            api_key="",
            secret_key="",
        )(
            root_order_id=root_order_id,
            client_order_id=client_order_id,
            limit_price=limit_price,
            stop_price=stop_price,
        )

    def create_response(
        self,
        data: Optional[list[dict]] = None,
        status_code: int = 200,
        text: Optional[str] = None,
    ) -> MagicMock:
        if data is None:
            data = [self.create_order_data()]

        return super().create_response(
            data=data,
            status_code=status_code,
            text=text,
        )

    def create_order_data(
        self,
        root_order_id: int = 123456789,
        client_order_id: Optional[str] = "abc123",
        order_id: int = 123456789,
        symbol: str = "USD_JPY",
        side: str = "BUY",
        order_type: str = "OCO",
        execution_type: str = "LIMIT",
        settle_type: str = "OPEN",
        size: int = 100,
        price: Optional[float] = 130.5,
        status: str = "ORDERED",
        cancel_type: Optional[str] = "OCO",
        expiry: str = "20220113",
        timestamp: str = "2019-03-19T02:15:06.059Z",
    ) -> dict:
        data = {
            "rootOrderId": root_order_id,
            "orderId": order_id,
            "symbol": symbol,
            "side": side,
            "orderType": order_type,
            "executionType": execution_type,
            "settleType": settle_type,
            "size": str(size),
            "status": status,
            "expiry": expiry,
            "timestamp": timestamp,
        }

        if client_order_id is not None:
            data["clientOrderId"] = client_order_id

        if price is not None:
            data["price"] = str(price)

        if cancel_type is not None:
            data["cancelType"] = cancel_type

        return data

    @patch("gmo_fx.api.api_base.post")
    def test_404_error(self, post_mock: MagicMock):
        self.check_404_error(post_mock, lambda: self.call_api(root_order_id=123456789))

    @patch("gmo_fx.api.api_base.post")
    def test_check_url(
        self,
        post_mock: MagicMock,
    ) -> None:
        post_mock.return_value = self.create_response()
        self.call_api(root_order_id=123456789)
        url: str = post_mock.mock_calls[0].args[0]
        assert url.startswith("https://forex-api.coin.z.com/private/v1/changeOcoOrder")

    def check_parse_a_data(self, post_mock: MagicMock, **kwargs) -> Order:
        post_mock.return_value = self.create_response(
            data=[self.create_order_data(**kwargs)]
        )
        response = self.call_api(root_order_id=123456789)
        assert len(response.orders) == 1
        return response.orders[0]

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_root_order_id(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, root_order_id=2536464541)
        assert order.root_order_id == 2536464541

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_client_order_id(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, client_order_id="abbb324dff")
        assert order.client_order_id == "abbb324dff"

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_none_without_client_order_id(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, client_order_id=None)
        assert order.client_order_id is None

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_order_id(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, order_id=156789)
        assert order.order_id == 156789

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_symbol(self, post_mock: MagicMock):
        for symbol in Order.Symbol:
            order = self.check_parse_a_data(post_mock, symbol=symbol.value)
            assert order.symbol == symbol

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_side(self, post_mock: MagicMock):
        for side in Order.Side:
            order = self.check_parse_a_data(post_mock, side=side.value)
            assert order.side == side

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_order_type(self, post_mock: MagicMock):
        for order_type in Order.OrderType:
            order = self.check_parse_a_data(post_mock, order_type=order_type.value)
            assert order.order_type == order_type

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_execution_type(self, post_mock: MagicMock):
        for execution_type in Order.ExecutionType:
            order = self.check_parse_a_data(
                post_mock, execution_type=execution_type.value
            )
            assert order.execution_type == execution_type

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_settle_type(self, post_mock: MagicMock):
        for settle_type in Order.SettleType:
            order = self.check_parse_a_data(post_mock, settle_type=settle_type.value)
            assert order.settle_type == settle_type

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_size(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, size=6100)
        assert order.size == 6100

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_price(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, price=155.44)
        assert order.price == 155.44

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_none_without_price(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, price=None)
        assert order.price is None

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_status(self, post_mock: MagicMock):
        for status in Order.Status:
            order = self.check_parse_a_data(post_mock, status=status.value)
            assert order.status == status

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_cancel_type(self, post_mock: MagicMock):
        for cancel_type in Order.CancelType:
            order = self.check_parse_a_data(post_mock, cancel_type=cancel_type.value)
            assert order.cancel_type == cancel_type

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_none_without_cancel_type(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, cancel_type=None)
        assert order.cancel_type is None

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_expiry(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, expiry="20190418")
        assert order.expiry == datetime(2019, 4, 18).date()

    @patch("gmo_fx.api.api_base.post")
    def test_should_get_timestamp(self, post_mock: MagicMock):
        order = self.check_parse_a_data(post_mock, timestamp="2024-09-13T15:21:03.059Z")
        assert order.timestamp == datetime(
            2024, 9, 13, 15, 21, 3, 59000, tzinfo=timezone.utc
        )

    @patch("gmo_fx.api.api_base.post")
    def test_should_parse_some_data(self, post_mock: MagicMock):
        post_mock.return_value = self.create_response(
            data=[self.create_order_data(), self.create_order_data()]
        )
        response = self.call_api(root_order_id=123456789)
        assert len(response.orders) == 2

    def check_call_with(self, post_mock, **kwargs):
        post_mock.return_value = self.create_response()
        # Ensure at least root_order_id or client_order_id is provided
        if "root_order_id" not in kwargs and "client_order_id" not in kwargs:
            kwargs["root_order_id"] = 123456789
        self.call_api(**kwargs)
        kall = post_mock.call_args
        request_body = kall.kwargs["data"]
        return json.loads(request_body)

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_with_root_order_id(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(post_mock, root_order_id=987654321)
        assert body["rootOrderId"] == 987654321

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_without_root_order_id(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(
            post_mock, root_order_id=None, client_order_id="test123"
        )
        assert "rootOrderId" not in body.keys()

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_with_client_order_id(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(post_mock, client_order_id="alfsdj32432enl")
        assert body["clientOrderId"] == "alfsdj32432enl"

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_without_client_order_id(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(
            post_mock, client_order_id=None, root_order_id=123456789
        )
        assert "clientOrderId" not in body.keys()

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_with_limit_price(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(post_mock, limit_price=155.66)
        assert body["limitPrice"] == "155.66"

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_without_limit_price(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(post_mock, limit_price=None, stop_price=154.67)
        assert "limitPrice" not in body.keys()

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_with_stop_price(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(post_mock, stop_price=154.67)
        assert body["stopPrice"] == "154.67"

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_without_stop_price(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(post_mock, stop_price=None, limit_price=155.66)
        assert "stopPrice" not in body.keys()

    @patch("gmo_fx.api.api_base.post")
    def test_should_call_api_with_both_prices(
        self,
        post_mock: MagicMock,
    ) -> None:
        body = self.check_call_with(post_mock, limit_price=140.525, stop_price=145.607)
        assert body["limitPrice"] == "140.525"
        assert body["stopPrice"] == "145.607"



from abc import ABC
from typing import Any, Callable, Optional, Union
import pytest
import json
from unittest.mock import MagicMock
from datetime import datetime


class ApiTestBase(ABC):

    def create_response(
        self,
        data: Optional[Union[dict, list]] = None,
        status_code: int = 200,
        text: Optional[str] = None,
    ) -> MagicMock:
        response = MagicMock()
        response.status_code = status_code
        json_data = {
            "status": 0,
            "data": data,
            "responsetime": datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
        }
        response.json.return_value = json_data
        response.text = text or json.dumps(json_data)
        return response

    def check_404_error(self, request_mock: MagicMock, target: Callable[[], Any]):
        request_mock.return_value = self.create_response(
            status_code=404,
            text="Not Found",
        )
        with pytest.raises(RuntimeError):
            target()



from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Literal
from requests import get, Response
from gmo_fx.api.api_base import PublicApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import Symbol
from gmo_fx.urls import BASE_URL_PUBLIC


@dataclass
class Kline:
    open_time: datetime
    open: float
    high: float
    low: float
    close: float


class KlinesResponse(ResponseBase):
    klines: list[Kline]

    def __init__(self, response: dict):
        super().__init__(response)
        self.klines = []

        data = response["data"]
        self.klines = [
            Kline(
                open=float(d["open"]),
                high=float(d["high"]),
                low=float(d["low"]),
                close=float(d["close"]),
                open_time=datetime.fromtimestamp(int(d["openTime"]) / 1000),
            )
            for d in data
        ]


class KlinesApi(PublicApiBase):
    Symbol = Symbol

    class KlineInterval(Enum):
        Min1 = "1min"
        Min5 = "5min"
        Min10 = "10min"
        Min15 = "15min"
        Min30 = "30min"
        H1 = "1hour"
        H4 = "4hour"
        H8 = "8hour"
        H12 = "12hour"
        D1 = "1day"
        W1 = "1week"
        M1 = "1month"

    @property
    def _path(self) -> str:
        return f"klines"

    @property
    def _method(self) -> PublicApiBase._HttpMethod:
        return self._HttpMethod.GET

    def __call__(
        self,
        symbol: Symbol,
        price_type: Literal["BID", "ASK"],
        interval: KlineInterval,
        date: datetime,
    ) -> KlinesResponse:
        date_str = f"{date.year:04}"
        if interval in (
            self.KlineInterval.Min1,
            self.KlineInterval.Min5,
            self.KlineInterval.Min10,
            self.KlineInterval.Min15,
            self.KlineInterval.Min30,
            self.KlineInterval.H1,
        ):
            date_str += f"{date.month:02}{date.day:02}"

        return super().__call__(
            path_query=f"symbol={symbol.value}"
            f"&priceType={price_type}"
            f"&interval={interval.value}"
            f"&date={date_str}"
        )

    @property
    def _response_parser(self):
        return KlinesResponse

    def _api_error_message(self, response: Response):
        return (
            "Klineが取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )



from typing import Callable, Optional
from unittest.mock import MagicMock, patch
from gmo_fx.api.symbols import Rule, SymbolsApi, SymbolsResponse

from tests.api_test_base import ApiTestBase


class TestSymbolsApi(ApiTestBase):
    SYMBOLS = [
        "USD_JPY",
        "EUR_JPY",
        "GBP_JPY",
        "AUD_JPY",
        "NZD_JPY",
        "CAD_JPY",
        "CHF_JPY",
        "TRY_JPY",
        "ZAR_JPY",
        "MXN_JPY",
        "EUR_USD",
        "GBP_USD",
        "AUD_USD",
        "NZD_USD",
    ]

    SYMBOLS_TABLE = {
        Rule.Symbol.USD_JPY: "USD_JPY",
        Rule.Symbol.EUR_JPY: "EUR_JPY",
        Rule.Symbol.GBP_JPY: "GBP_JPY",
        Rule.Symbol.AUD_JPY: "AUD_JPY",
        Rule.Symbol.NZD_JPY: "NZD_JPY",
        Rule.Symbol.CAD_JPY: "CAD_JPY",
        Rule.Symbol.CHF_JPY: "CHF_JPY",
        Rule.Symbol.TRY_JPY: "TRY_JPY",
        Rule.Symbol.ZAR_JPY: "ZAR_JPY",
        Rule.Symbol.MXN_JPY: "MXN_JPY",
        Rule.Symbol.EUR_USD: "EUR_USD",
        Rule.Symbol.GBP_USD: "GBP_USD",
        Rule.Symbol.AUD_USD: "AUD_USD",
        Rule.Symbol.NZD_USD: "NZD_USD",
    }

    def call_api(
        self,
    ) -> SymbolsResponse:
        return SymbolsApi()()

    def create_symbol_data(
        self,
        symbol: str = "USD_JPY",
        min_open_order_size: int = 0,
        max_order_size: int = 0,
        size_step: int = 0,
        tick_size: float = 0.0,
    ) -> dict:
        return {
            "symbol": symbol,
            "minOpenOrderSize": str(min_open_order_size),
            "maxOrderSize": str(max_order_size),
            "sizeStep": str(size_step),
            "tickSize": str(tick_size),
        }

    def create_symbols_data(
        self, symbols_data_builder: Optional[Callable[[str], dict]] = None
    ) -> list[dict]:
        symbols_data_builder = symbols_data_builder or (
            lambda symbol: self.create_symbol_data(symbol=symbol)
        )
        return [symbols_data_builder(symbol) for symbol in self.SYMBOLS]

    @patch("gmo_fx.api.api_base.get")
    def test_symbols_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_symbols_from_symbols(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(data=self.create_symbols_data())
        symbols_response = self.call_api()
        symbols = [rule.symbol for rule in symbols_response.rules]
        for symbol in Rule.Symbol:
            assert symbol in symbols
            symbols.remove(symbol)
        else:
            assert len(symbols) == 0

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_min_open_order_size_from_symbols(self, get_mock: MagicMock):
        def fixture_min_open_order_size(symbol: str):
            return self.SYMBOLS.index(symbol) * 100

        def fixture_symbols_data(symbol: str) -> dict:
            size = fixture_min_open_order_size(symbol)
            return self.create_symbol_data(symbol, min_open_order_size=size)

        get_mock.return_value = self.create_response(
            data=self.create_symbols_data(symbols_data_builder=fixture_symbols_data)
        )
        rules = self.call_api().rules
        for rule in rules:
            assert rule.min_open_order_size == fixture_min_open_order_size(
                self.SYMBOLS_TABLE[rule.symbol]
            )

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_max_order_size_from_symbols(self, get_mock: MagicMock):
        def fixture_max_order_size(symbol: str):
            return self.SYMBOLS.index(symbol) * 100

        def fixture_symbols_data(symbol: str) -> dict:
            size = fixture_max_order_size(symbol)
            return self.create_symbol_data(symbol, max_order_size=size)

        get_mock.return_value = self.create_response(
            data=self.create_symbols_data(symbols_data_builder=fixture_symbols_data)
        )
        rules = self.call_api().rules
        for rule in rules:
            assert rule.max_order_size == fixture_max_order_size(
                self.SYMBOLS_TABLE[rule.symbol]
            )

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_size_step_from_symbols(self, get_mock: MagicMock):
        def fixture_size_step(symbol: str):
            return self.SYMBOLS.index(symbol) * 100

        def fixture_symbols_data(symbol: str) -> dict:
            step = fixture_size_step(symbol)
            return self.create_symbol_data(symbol, size_step=step)

        get_mock.return_value = self.create_response(
            data=self.create_symbols_data(symbols_data_builder=fixture_symbols_data)
        )
        rules = self.call_api().rules
        for rule in rules:
            assert rule.size_step == fixture_size_step(self.SYMBOLS_TABLE[rule.symbol])

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_tick_size_from_symbols(self, get_mock: MagicMock):
        def fixture_tick_size(symbol: str):
            return (self.SYMBOLS.index(symbol) / 3) * 100

        def fixture_symbols_data(symbol: str) -> dict:
            size = fixture_tick_size(symbol)
            return self.create_symbol_data(symbol, tick_size=size)

        get_mock.return_value = self.create_response(
            data=self.create_symbols_data(symbols_data_builder=fixture_symbols_data)
        )
        rules = self.call_api().rules
        for rule in rules:
            assert rule.tick_size == fixture_tick_size(self.SYMBOLS_TABLE[rule.symbol])

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(data=self.create_symbols_data())
        self.call_api()
        url = get_mock.mock_calls[0].args[0]
        assert url == "https://forex-api.coin.z.com/public/v1/symbols"



import re

from typing import Callable, Optional
from unittest.mock import MagicMock, patch
from gmo_fx.common import Symbol
from gmo_fx.api.assets import AssetsApi, AssetsResponse

from tests.api_test_base import ApiTestBase


class TestAssetsApi(ApiTestBase):

    def call_api(
        self,
    ) -> AssetsResponse:
        return AssetsApi(
            api_key="",
            secret_key="",
        )()

    def create_assets_data(
        self,
        equity: int = 0,
        available_amount: int = 0,
        balance: int = 0,
        estimated_trade_fee: float = 0.0,
        margin: int = 0,
        margin_ratio: float = 0.0,
        position_loss_gain: float = 0.0,
        total_swap: float = 0.0,
        transferable_amount: int = 0,
    ) -> dict:
        return [
            {
                "equity": equity,
                "availableAmount": available_amount,
                "balance": balance,
                "estimatedTradeFee": estimated_trade_fee,
                "margin": margin,
                "marginRatio": margin_ratio,
                "positionLossGain": position_loss_gain,
                "totalSwap": total_swap,
                "transferableAmount": transferable_amount,
            }
        ]

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_equity(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(equity=1)
        )
        response = self.call_api()
        equities = [asset.equity for asset in response.assets]
        assert equities[0] == 1

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_available_amount(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(available_amount=1)
        )
        response = self.call_api()
        available_amounts = [asset.available_amount for asset in response.assets]
        assert available_amounts[0] == 1

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_balance(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(balance=1)
        )
        response = self.call_api()
        balances = [asset.balance for asset in response.assets]
        assert balances[0] == 1

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_estimated_trade_fee(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(estimated_trade_fee=1.2)
        )
        response = self.call_api()
        estimated_trade_fees = [asset.estimated_trade_fee for asset in response.assets]
        assert estimated_trade_fees[0] == 1.2

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_margin(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(margin=1)
        )
        response = self.call_api()
        margins = [asset.margin for asset in response.assets]
        assert margins[0] == 1

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_margin_ratio(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(margin_ratio=1.4)
        )
        response = self.call_api()
        margin_ratios = [asset.margin_ratio for asset in response.assets]
        assert margin_ratios[0] == 1.4

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_loss_gain(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(position_loss_gain=1.4)
        )
        response = self.call_api()
        position_loss_gains = [asset.position_loss_gain for asset in response.assets]
        assert position_loss_gains[0] == 1.4

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_total_swap(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(total_swap=1.4)
        )
        response = self.call_api()
        total_swaps = [asset.total_swap for asset in response.assets]
        assert total_swaps[0] == 1.4

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_transferable_amount(self, get_mock: MagicMock):
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(transferable_amount=1)
        )
        response = self.call_api()
        transferable_amounts = [asset.transferable_amount for asset in response.assets]
        assert transferable_amounts[0] == 1

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response(
            data=self.create_assets_data(equity=1)
        )
        self.call_api()
        url = get_mock.mock_calls[0].args[0]
        assert url == "https://forex-api.coin.z.com/private/v1/account/assets"



from typing import Optional


class ApiError(Exception):
    error_code: str

    __ERR_MAP = {
        "ERR-143": "規制中のため取引できません。",
        "ERR-148": "IFD、IFDOCO注文において決済注文数量が新規注文数量を超えています。",
        "ERR-189": "決済注文において、決済可能保有数量を超えての数量は注文できません。",
        "ERR-200": "すでに有効注文があり、注文数量が発注可能な数量を超えています。決済注文を行いたい場合は、注文数量を変更するか、有効注文を取消したうえで、再度注文をしてください。",
        "ERR-201": "取引余力が不足しています。",
        "ERR-254": "指定された建玉が存在しません。",
        "ERR-414": "決済注文において指定した銘柄と指定した建玉IDの銘柄が異なっています。",
        "ERR-423": "決済注文において注文数量が(指定した建玉、もしくは指定銘柄の建玉の)総建玉数量よりも大きい、もしくは総有効注文数量との合計が総建玉数量よりも大きいです。",
        "ERR-512": "決済注文において同じ建玉IDが2つ以上指定されました。",
        "ERR-542": "OCO注文変更においてOCO注文以外の親注文IDや顧客注文IDが指定されました。",
        "ERR-595": "IFD、IFDOCO注文変更においてIFD、IFDOCO注文以外の親注文IDや顧客注文IDが指定されました。",
        "ERR-635": "有効注文の件数が上限に達しています。新規の注文をするためには、有効注文を取消したうえで、再度注文をしてください。",
        "ERR-759": "建玉の件数が上限に達している場合、新規注文は行えません。",
        "ERR-760": "注文変更において変更前と後で注文レートが同じです。",
        "ERR-761": "指定可能な注文レートの範囲を超えています。",
        "ERR-769": "注文変更において変更元となる注文の有効期限が切れています。",
        "ERR-786": "設定した顧客注文IDがご自身の他の有効注文に使用されています。",
        "ERR-5003": "API呼出上限を越えています。",
        "ERR-5008": "リクエストヘッダーに設定されているAPI-TIMESTAMPが公開APIのシステム時刻より遅いです。システム時間の更新をお試しください。",
        "ERR-5009": "リクエストヘッダーに設定されているAPI-TIMESTAMPが公開APIのシステム時刻より早いです。システム時間の更新をお試しください。",
        "ERR-5010": "リクエストヘッダーに指定されているAPI-SIGN（Signature）に不正があります。認証情報についてご確認ください。",
        "ERR-5011": "リクエストヘッダーにAPI-KEYが設定されていません。認証情報についてご確認ください。",
        "ERR-5012": "API-KEYの認証エラーが発生しました。",
        "ERR-5014": "お客様のご確認、ご同意が必要な処理を実行する際に、お客様に未確認・未同意事項がありました。",
        "ERR-5106": "パラメーターが不正です。",
        "ERR-5114": "注文レートが呼値の単位を超えています。取引ルールの呼値をご確認ください。取引ルールは以下よりご確認いただけます。\nhttps://coin.z.com/jp/corp/product/info/fx/?=rule",
        "ERR-5122": "指定された注文がすでに変更中、取消中、取消済、全量約定、失効のいずれかの状態であるため、注文の変更や取消ができません。",
        "ERR-5123": "指定された注文番号が存在しません。",
        "ERR-5125": "APIの取引規制がかかっています。",
        "ERR-5126": "1回あたりの最大注文数量を超えているもしくは最小注文数量を下回っている、または最大建玉数量を超過しています。以下より取引ルールをご確認ください。\nhttps://coin.z.com/jp/corp/product/info/fx/?=rule",
        "ERR-5130": "注文情報取得もしくは約定情報取得において指定した注文IDもしくは親注文IDが最大指定数量を超えています。",
        "ERR-5132": "約定情報取得において指定した約定IDが最大指定数量を超えています。",
        "ERR-5201": "定期メンテナンス中です。",
        "ERR-5202": "緊急メンテナンス中です。",
        "ERR-5204": "APIのURLが不正です。",
        "ERR-5206": "注文変更を実行時に注文毎の変更可能回数が上限に達しています。上限に達して更に注文変更を行いたい場合は、1度注文キャンセルをしてから新規に注文をしていただくようにお願い致します。",
        "ERR-5207": "リクエストボディに指定されているsymbol、intervalまたはdateが不正です。",
        "ERR-5208": "約定情報取得において、orderId/executionIdが両方設定されました。",
        "ERR-5209": "注文情報取得において、orderId/rootOrderIdが両方設定されました。",
        "ERR-5210": "注文タイプと limitPrice/stopPriceが一致しません。",
        "ERR-5211": "売買区分と一致しない lowerBound/upperBound が指定された、もしくは指値や逆指値注文に lowerBound/upperBound が指定されました。",
        "ERR-5213": "注文IDもしくは親注文IDと顧客注文IDどちらか一方を指定する箇所でどちらも指定されました。",
        "ERR-5214": "決済注文において、建玉IDを指定している場合に注文数量が指定されました。",
        "ERR-5218": "取引時間外のため注文が行えません。",
        "ERR-5219": "注文変更において通常注文以外の注文IDもしくは、顧客注文IDが指定されました。",
        "ERR-5220": "IFD注文において1次注文レートを2次注文レートより低く設定しなければなりません。",
        "ERR-5221": "IFD注文において1次注文レートを2次注文レートより高く設定しなければなりません。",
        "ERR-5222": "IFDOCO注文において1次注文レートを2次指値注文レートより低く設定しなければなりません。",
        "ERR-5223": "IFDOCO注文において1次注文レートを2次指値注文レートより高く設定しなければなりません。",
        "ERR-5224": "IFDOCO注文において1次注文レートを2次逆指値注文レートより高く設定しなければなりません。",
        "ERR-5225": "IFDOCO注文において1次注文レートを2次逆指値注文レートより低く設定しなければなりません。",
        "ERR-5226": "逆指値の買い注文において、注文レートが現在レートより低いです。",
        "ERR-5227": "逆指値の売り注文において、注文レートが現在レートより高いです。",
        "ERR-5228": "OCO売り注文において逆指値注文レートが指値注文レートを超えています。",
        "ERR-5229": "OCO買い注文において逆指値注文レートが指値注文レートを下回っています。",
    }

    def __init__(
        self, error_code: str, *args: object, message: Optional[str] = None
    ) -> None:
        super().__init__(*args)
        self.error_code = error_code
        self.message = message

    def __str__(self):
        message = self.__ERR_MAP[self.error_code]
        if self.message:
            message += f"\nmessage: {self.message}"
        return message



from dataclasses import dataclass
from datetime import date, datetime, timezone
from enum import Enum
from typing import Optional
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase

from gmo_fx.common import (
    OrderType,
    SettleType,
    Side,
    Symbol,
)
from gmo_fx.api.response import Response as ResponseBase


@dataclass
class CloseOrder:

    class Status(Enum):
        WAITING = "WAITING"
        EXECUTED = "EXECUTED"
        EXPIRED = "EXPIRED"

    class CancelType(Enum):
        PRICE_BOUND = "PRICE_BOUND"
        OCO = "OCO"

    class ExecutionType(Enum):
        """注文タイプ"""

        MARKET = "MARKET"
        LIMIT = "LIMIT"
        STOP = "STOP"

    OrderType = OrderType
    SettleType = SettleType
    Side = Side
    Symbol = Symbol

    root_order_id: int
    client_order_id: Optional[str]
    order_id: int
    symbol: Symbol
    side: Side
    order_type: OrderType
    execution_type: ExecutionType
    settle_type: SettleType
    size: int
    price: Optional[float]
    status: Status
    cancel_type: Optional[CancelType]
    expiry: Optional[date]
    timestamp: datetime


class CloseOrderResponse(ResponseBase):
    close_orders: list[CloseOrder]

    def __init__(self, response: dict):
        super().__init__(response)
        self.close_orders = []

        data: list[dict] = response["data"]
        self.close_orders = [
            CloseOrder(
                root_order_id=d["rootOrderId"],
                client_order_id=d.get("clientOrderId"),
                order_id=d["orderId"],
                symbol=CloseOrder.Symbol(d["symbol"]),
                side=CloseOrder.Side(d["side"]),
                order_type=CloseOrder.OrderType(d["orderType"]),
                execution_type=CloseOrder.ExecutionType(d["executionType"]),
                settle_type=CloseOrder.SettleType(d["settleType"]),
                size=int(d["size"]),
                price=float(d.get("price")) if d.get("price") else None,
                status=CloseOrder.Status(d["status"]),
                cancel_type=(
                    CloseOrder.CancelType(d.get("cancelType"))
                    if d.get("cancelType")
                    else None
                ),
                expiry=(
                    datetime.strptime(d["expiry"], "%Y%m%d").date()
                    if d.get("expiry")
                    else None
                ),
                timestamp=datetime.strptime(
                    d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"
                ).replace(tzinfo=timezone.utc),
            )
            for d in data
        ]


class CloseOrderApi(PrivateApiBase):
    Symbol = Symbol
    Side = Side

    class ExecutionType(Enum):
        """注文タイプ"""

        MARKET = "MARKET"
        LIMIT = "LIMIT"
        STOP = "STOP"
        OCO = "OCO"

    @dataclass
    class SettlePosition:
        position_id: int
        size: int

        def to_dict(self) -> dict:
            return {
                "positionId": self.position_id,
                "size": str(self.size),
            }

    @property
    def _path(self) -> str:
        return "closeOrder"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.POST

    @property
    def _response_parser(self):
        return CloseOrderResponse

    def _api_error_message(self, response: Response):
        return (
            "決済注文が失敗しました\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def __call__(
        self,
        symbol: Symbol,
        side: Side,
        execution_type: ExecutionType,
        client_order_id: Optional[str] = None,
        size: Optional[int] = None,
        limit_price: Optional[float] = None,
        stop_price: Optional[float] = None,
        lower_bound: Optional[float] = None,
        upper_bound: Optional[float] = None,
        settle_position: Optional[list[SettlePosition]] = None,
    ) -> CloseOrderResponse:
        data = {
            "symbol": symbol.value,
            "side": side.value,
            "executionType": execution_type.value,
        }

        if client_order_id is not None:
            data["clientOrderId"] = client_order_id

        if size is not None:
            data["size"] = str(size)

        if limit_price is not None:
            data["limitPrice"] = str(limit_price)

        if stop_price is not None:
            data["stopPrice"] = str(stop_price)

        if lower_bound is not None:
            data["lowerBound"] = str(lower_bound)

        if upper_bound is not None:
            data["upperBound"] = str(upper_bound)

        if settle_position is not None:
            data["settlePosition"] = [settle.to_dict() for settle in settle_position]

        return super().__call__(
            data=data,
        )



import hmac
import hashlib
import json
import pytest
import time
import time_machine
from datetime import datetime
from api.api_base import PrivateApiBase
from api.response import Response
from typing import Any, Optional
from unittest.mock import MagicMock, patch


class PrivateApiBaseStub(PrivateApiBase):
    def __init__(
        self,
        api_key: str,
        secret_key: str,
        method: str = "GET",
        body: dict = {},
        path: str = "test",
        is_call: bool = False,
    ) -> None:
        self.__method = (
            self._HttpMethod.POST if method == "POST" else self._HttpMethod.GET
        )
        self.__path = path
        self.__body = body
        self.__is_call = is_call
        super().__init__(api_key, secret_key)

    def __call__(self, *args: Any, **kwds: Any) -> Response:
        if self.__is_call:
            return super().__call__(*args, **kwds)

    @property
    def _response_parser(self):
        return MagicMock()

    @property
    def _path(
        self,
    ) -> str:
        return self.__path

    @property
    def _body(
        self,
    ) -> str:
        return self.__body

    @property
    def _method(self):
        return self.__method

    @property
    def header(
        self,
    ) -> dict:
        return super()._create_header()


class TestPrivateApiBase:

    def test_api_key_header(self):
        header = PrivateApiBaseStub(api_key="api_key", secret_key="secret").header
        assert header["API-KEY"] == "api_key"

    bodys = [
        # (body, expect),
        (None, None),
        ({}, "{}"),
        ({"test": "test"}, '{"test":"test"}'),
        ({"test": ""}, '{"test":""}'),
    ]

    @pytest.mark.parametrize(
        "body, expect",
        bodys,
    )
    @patch("api.api_base.post")
    def test_should_post_with_body(
        self,
        post_mock: MagicMock,
        body: Optional[dict],
        expect: Optional[str],
    ):
        respose = MagicMock()
        respose.status_code = 200
        respose.json.return_value = {}
        post_mock.return_value = respose
        api = PrivateApiBaseStub(
            api_key="api_key",
            secret_key="secret",
            body=body,
            method="POST",
            is_call=True,
        )
        api(data=body)
        post_mock.assert_called_once()
        kall = post_mock.call_args
        expect = (
            expect.replace(":", ": ") if expect else None
        )  # FIXME: 定義でスペースを空けるとテストがエラーになる
        assert kall.kwargs["data"] == expect
        assert api._body == body

    @time_machine.travel(datetime(2022, 12, 25))
    def test_timestamp(self):
        header = PrivateApiBaseStub(api_key="api_key", secret_key="secret").header
        assert header["API-TIMESTAMP"] == "{0}000".format(
            int(time.mktime(datetime.now().timetuple()))
        )

    @time_machine.travel(datetime(2024, 1, 3))
    def test_api_sign_get(self):
        method = "GET"
        path = "test"
        body = {"a": "a"}
        header = PrivateApiBaseStub(
            api_key="api_key", secret_key="secret", method=method, path=path, body=body
        ).header
        timestamp = "{0}000".format(int(time.mktime(datetime.now().timetuple())))
        text = timestamp + method + f"/{PrivateApiBaseStub.VERSION}/{path}"
        assert (
            header["API-SIGN"]
            == hmac.new(
                bytes("secret".encode("ascii")),
                bytes(text.encode("ascii")),
                hashlib.sha256,
            ).hexdigest()
        )

    @time_machine.travel(datetime(2024, 1, 3))
    def test_api_sign_post(self):
        method = "POST"
        path = "test"
        body = {"a": "a"}
        header = PrivateApiBaseStub(
            api_key="api_key", secret_key="secret", method=method, path=path, body=body
        ).header
        timestamp = "{0}000".format(int(time.mktime(datetime.now().timetuple())))
        text = (
            timestamp
            + method
            + f"/{PrivateApiBaseStub.VERSION}/{path}"
            + json.dumps(body)
        )
        assert (
            header["API-SIGN"]
            == hmac.new(
                bytes("secret".encode("ascii")),
                bytes(text.encode("ascii")),
                hashlib.sha256,
            ).hexdigest()
        )



from dataclasses import dataclass
from datetime import date, datetime, timezone
from enum import Enum
from typing import Optional
from requests import Response
from gmo_fx.api.api_base import PrivateApiBase

from gmo_fx.common import (
    OrderType,
    SettleType,
    Side,
    Symbol,
)
from gmo_fx.api.response import Response as ResponseBase


@dataclass
class Order:

    class Status(Enum):
        ORDERED = "ORDERED"
        MODIFYING = "MODIFYING"
        EXECUTED = "EXECUTED"
        EXPIRED = "EXPIRED"

    class CancelType(Enum):
        """取り消し区分"""

        OCO = "OCO"

    class ExecutionType(Enum):
        """注文タイプ"""

        LIMIT = "LIMIT"
        STOP = "STOP"

    OrderType = OrderType
    SettleType = SettleType
    Side = Side
    Symbol = Symbol

    root_order_id: int
    client_order_id: Optional[str]
    order_id: int
    symbol: Symbol
    side: Side
    order_type: OrderType
    execution_type: ExecutionType
    settle_type: SettleType
    size: int
    price: Optional[float]
    status: Status
    cancel_type: Optional[CancelType]
    expiry: date
    timestamp: datetime


class ChangeOcoOrderResponse(ResponseBase):
    orders: list[Order]

    def __init__(self, response: dict):
        super().__init__(response)
        self.orders = []

        data: list[dict] = response["data"]
        self.orders = [
            Order(
                root_order_id=d["rootOrderId"],
                client_order_id=d.get("clientOrderId"),
                order_id=d["orderId"],
                symbol=Order.Symbol(d["symbol"]),
                side=Order.Side(d["side"]),
                order_type=Order.OrderType(d["orderType"]),
                execution_type=Order.ExecutionType(d["executionType"]),
                settle_type=Order.SettleType(d["settleType"]),
                size=int(d["size"]),
                price=float(d.get("price")) if d.get("price") else None,
                status=Order.Status(d["status"]),
                cancel_type=(
                    Order.CancelType(d.get("cancelType"))
                    if d.get("cancelType")
                    else None
                ),
                expiry=datetime.strptime(d["expiry"], "%Y%m%d").date(),
                timestamp=datetime.strptime(
                    d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"
                ).replace(tzinfo=timezone.utc),
            )
            for d in data
        ]


class ChangeOcoOrderApi(PrivateApiBase):
    @property
    def _path(self) -> str:
        return "changeOcoOrder"

    @property
    def _method(self) -> PrivateApiBase._HttpMethod:
        return self._HttpMethod.POST

    @property
    def _response_parser(self):
        return ChangeOcoOrderResponse

    def _api_error_message(self, response: Response):
        return (
            "OCO注文変更が失敗しました\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )

    def __call__(
        self,
        root_order_id: Optional[int] = None,
        client_order_id: Optional[str] = None,
        limit_price: Optional[float] = None,
        stop_price: Optional[float] = None,
    ) -> ChangeOcoOrderResponse:
        if root_order_id is None and client_order_id is None:
            raise ValueError("root_order_id or client_order_id must be provided")

        data: dict = {}

        if root_order_id is not None:
            data["rootOrderId"] = root_order_id

        if client_order_id is not None:
            data["clientOrderId"] = client_order_id

        if limit_price is not None:
            data["limitPrice"] = str(limit_price)

        if stop_price is not None:
            data["stopPrice"] = str(stop_price)

        return super().__call__(
            data=data,
        )



from setuptools import setup, find_packages

DESCRIPTION = "GMO社のFX API用Pythonライブラリ"
NAME = "gmo_fx"
AUTHOR = "Rikito Noto"
AUTHOR_EMAIL = "rikitonoto@gmail.com"
URL = "https://github.com/RikitoNoto/gmo-fx-py"
LICENSE = "MIT"
DOWNLOAD_URL = "https://github.com/RikitoNoto/gmo-fx-py"
VERSION = "0.8.0"
INSTALL_REQUIRES = [
    "requests>=2.31.0",
]

with open("README.md", "r") as fp:
    readme = fp.read()

setup(
    name=NAME,
    description=DESCRIPTION,
    long_description=readme,
    long_description_content_type="text/markdown",
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license=LICENSE,
    download_url=DOWNLOAD_URL,
    version=VERSION,
    install_requires=INSTALL_REQUIRES,
    packages=find_packages(),
)



from dataclasses import dataclass
from datetime import datetime
from requests import get, Response
from gmo_fx.api.api_base import PublicApiBase
from gmo_fx.api.response import Response as ResponseBase
from gmo_fx.common import Symbol
from gmo_fx.urls import BASE_URL_PUBLIC


@dataclass
class Ticker:
    Symbol = Symbol
    symbol: Symbol
    ask: float
    bid: float
    timestamp: datetime
    status: str


class TickerResponse(ResponseBase):
    tickers: list[Ticker]

    def __init__(self, response: dict):
        super().__init__(response)
        self.tickers = []

        data = response["data"]
        self.tickers = [
            Ticker(
                symbol=Ticker.Symbol(d["symbol"]),
                ask=d["ask"],
                bid=d["bid"],
                status=d["status"],
                timestamp=datetime.strptime(d["timestamp"], "%Y-%m-%dT%H:%M:%S.%fZ"),
            )
            for d in data
        ]


class TickerApi(PublicApiBase):

    @property
    def _path(self) -> str:
        return f"ticker"

    @property
    def _method(self) -> PublicApiBase._HttpMethod:
        return self._HttpMethod.GET

    @property
    def _response_parser(self):
        return TickerResponse

    def _api_error_message(self, response: Response):
        return (
            "最新レートが取得できませんでした。\n"
            f"status code: {response.status_code}\n"
            f"response: {response.text}"
        )



from datetime import datetime, timezone
from typing import Optional
from unittest.mock import MagicMock, patch
from gmo_fx.api.open_positions import (
    OpenPosition,
    OpenPositionsApi,
    OpenPositionsResponse,
)

from tests.api_test_base import ApiTestBase


class TestOpenPositionApi(ApiTestBase):

    def call_api(
        self,
        symbol: Optional[OpenPositionsApi.Symbol] = None,
        prev_id: Optional[int] = None,
        count: Optional[int] = None,
    ) -> OpenPositionsResponse:
        return OpenPositionsApi(
            api_key="",
            secret_key="",
        )(symbol=symbol, prev_id=prev_id, count=count)

    def create_response(
        self,
        data: Optional[list[dict]] = None,
        status_code: int = 200,
        text: Optional[str] = None,
    ) -> MagicMock:
        if data is None:
            data = [self.create_open_position_data()]

        return super().create_response(
            data={"list": data},
            status_code=status_code,
            text=text,
        )

    def create_open_position_data(
        self,
        position_id: int = 0,
        symbol: str = "USD_JPY",
        side: str = "BUY",
        size: int = 100,
        ordered_size: int = 1,
        price: float = 100.5,
        loss_gain: float = 10.5,
        total_swap: float = 0.1,
        timestamp: str = datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%fZ"),
    ) -> dict:
        return {
            "positionId": position_id,
            "symbol": symbol,
            "side": side,
            "size": str(size),
            "orderedSize": str(ordered_size),
            "price": str(price),
            "lossGain": str(loss_gain),
            "totalSwap": str(total_swap),
            "timestamp": timestamp,
        }

    @patch("gmo_fx.api.api_base.get")
    def test_404_error(self, get_mock: MagicMock):
        self.check_404_error(get_mock, lambda: self.call_api())

    @patch("gmo_fx.api.api_base.get")
    def test_check_url(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api()
        url: str = get_mock.mock_calls[0].args[0]
        assert url.startswith("https://forex-api.coin.z.com/private/v1/openPositions")

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_symbol(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api(symbol=OpenPositionsApi.Symbol.GBP_USD)
        url = get_mock.mock_calls[0].args[0]
        assert "symbol=GBP_USD" in url

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_without_symbol(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api(symbol=None)
        url = get_mock.mock_calls[0].args[0]
        assert "symbol=" not in url

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_prev_id(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api(prev_id=12345)
        url = get_mock.mock_calls[0].args[0]
        assert "prevId=12345" in url

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_without_prev_id(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api(prev_id=None)
        url = get_mock.mock_calls[0].args[0]
        assert "prevId=" not in url

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_with_count(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api(count=12345)
        url = get_mock.mock_calls[0].args[0]
        assert "count=12345" in url

    @patch("gmo_fx.api.api_base.get")
    def test_should_call_api_without_count(
        self,
        get_mock: MagicMock,
    ) -> None:
        get_mock.return_value = self.create_response()
        self.call_api(count=None)
        url = get_mock.mock_calls[0].args[0]
        assert "count=" not in url

    def check_parse_a_data(
        self, get_mock: MagicMock, **kwargs
    ) -> OpenPositionsResponse:
        get_mock.return_value = self.create_response(
            data=[self.create_open_position_data(**kwargs)]
        )
        response = self.call_api()
        assert len(response.open_positions) == 1
        return response

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_position_id(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, position_id=123123)
        assert response.open_positions[0].position_id == 123123

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_symbol(self, get_mock: MagicMock):
        for symbol in OpenPosition.Symbol:
            response = self.check_parse_a_data(get_mock, symbol=symbol.value)
            assert response.open_positions[0].symbol == symbol

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_side(self, get_mock: MagicMock):
        for side in OpenPosition.Side:
            response = self.check_parse_a_data(get_mock, side=side.value)
            assert response.open_positions[0].side == side

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_size(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, size=130204)
        assert response.open_positions[0].size == 130204

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_ordersize(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, ordered_size=99864)
        assert response.open_positions[0].ordered_size == 99864

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_price(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, price=101.123)
        assert response.open_positions[0].price == 101.123

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_loss_gain(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, loss_gain=10500.66)
        assert response.open_positions[0].loss_gain == 10500.66

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_total_swap(self, get_mock: MagicMock):
        response = self.check_parse_a_data(get_mock, total_swap=166.13)
        assert response.open_positions[0].total_swap == 166.13

    @patch("gmo_fx.api.api_base.get")
    def test_should_get_timestamp(self, get_mock: MagicMock):
        response = self.check_parse_a_data(
            get_mock, timestamp="2019-03-21T05:18:09.011Z"
        )
        assert response.open_positions[0].timestamp == datetime(
            2019, 3, 21, 5, 18, 9, 11000, tzinfo=timezone.utc
        )
